@@ -24,6 +24,7 @@ from evince_dbus import EvinceWindowProxy
 import dbus.mainloop.glib
 import logging
 import gettext
+import os
 from gpdefs import *
 
 try:
@@ -47,7 +48,6 @@ ui_str = """<ui>
 """
 
 VIEW_DATA_KEY = "SynctexPluginViewData"
-WINDOW_DATA_KEY = "SynctexPluginWindowData"
 
 _logger = logging.getLogger("SynctexPlugin")
 
@@ -57,6 +57,7 @@ def apply_style (style, tag):
             tag.set_property(prop, style.get_property(prop))
         else:
             tag.set_property(prop, None)
+
     def apply_style_prop_bool(tag, style, prop, whentrue, whenfalse):
         if style.get_property(prop + "-set"):
             prop_value = whentrue if style.get_property(prop) else whenfalse
@@ -73,9 +74,11 @@ def apply_style (style, tag):
 
 def parse_modeline(line):
     index = line.find("mainfile:")
+
     if line.startswith("%") and index > 0:
         # Parse the modeline.
         return line[index + len("mainfile:"):].strip()
+
     return None
 
 class SynctexViewHelper:
@@ -85,6 +88,7 @@ class SynctexViewHelper:
         self._plugin = plugin
         self._doc = view.get_buffer()
         self.window_proxy = None
+
         self._handlers = [
             self._doc.connect('saved', self.on_saved_or_loaded),
             self._doc.connect('loaded', self.on_saved_or_loaded)
@@ -102,6 +106,7 @@ class SynctexViewHelper:
     def on_button_release(self, view, event):
         modifier_mask = Gtk.accelerator_get_default_mod_mask()
         event_state = event.state & modifier_mask
+
         if event.button == 1 and event_state == Gdk.ModifierType.CONTROL_MASK:
             self.sync_view(event.time)
 
@@ -111,6 +116,7 @@ class SynctexViewHelper:
     def get_output_file(self):
         file_output = None
         line_count = self._doc.get_line_count()
+
         for i in range(min(3,line_count)) + range(max(0,line_count - 3), line_count):
             start = self._doc.get_iter_at_line(i)
             end = start.copy()
@@ -118,27 +124,31 @@ class SynctexViewHelper:
             file_output = parse_modeline(self._doc.get_text(start, end, False))
             if file_output is not None:
                 break
+
         return file_output
 
     def on_key_press(self, a, b):
         self._unhighlight()
+
     def on_cursor_moved(self, cur):
         self._unhighlight()
 
     def deactivate(self):
         self._unhighlight()
+
         for h in self._handlers:
             self._doc.disconnect(h)
+
         del self._highlight_tag
 
     def update_location(self):
-        import os
-
         gfile = self._doc.get_location()
+
         if gfile is None:
             return
+
         if self.gfile is None or gfile.get_uri() != self.gfile.get_uri():
-            SynctexPlugin.view_dict[gfile.get_uri()] = self
+            SynctexWindowActivatable.view_dict[gfile.get_uri()] = self
             self.gfile = gfile
 
         modeline_output_file = self.get_output_file()
@@ -151,6 +161,7 @@ class SynctexViewHelper:
         out_path = self.gfile.get_parent().get_child(filename).get_path()
         out_path = os.path.splitext(out_path)
         out_gfile = Gio.file_new_for_path(out_path[0] + ".pdf")
+
         if out_gfile.query_exists(None):
             self.out_gfile = out_gfile
         else:
@@ -162,6 +173,7 @@ class SynctexViewHelper:
         iter = self._doc.get_iter_at_mark(self._doc.get_insert())
         end_iter = iter.copy()
         end_iter.forward_to_line_end()
+
         self._doc.apply_tag(self._highlight_tag, iter, end_iter)
         self.last_iters = [iter, end_iter];
 
@@ -179,7 +191,7 @@ class SynctexViewHelper:
         self._window.present_with_time (time)
 
     def goto_line_after_load(self, a, line, time):
-        GObject.idle_add(lambda : self.goto_line(line, time))
+        GObject.idle_add (lambda : self.goto_line(line, time))
         self._doc.disconnect(self._goto_handler)
 
     def sync_view(self, time):
@@ -191,7 +203,7 @@ class SynctexViewHelper:
 
     def update_active(self):
         # Activate the plugin only if the doc is a LaTeX file.
-        lang = self._doc.get_language() 
+        lang = self._doc.get_language()
         self.active = (lang is not None and lang.get_id() == 'latex' and
                         self.out_gfile is not None)
 
@@ -206,7 +218,7 @@ class SynctexViewHelper:
 
             style = self._doc.get_style_scheme().get_style('search-match')
             apply_style(style, self._highlight_tag)
-            self._window.get_data(WINDOW_DATA_KEY)._action_group.set_sensitive(True)
+            self._plugin._action_group.set_sensitive(True)
             self.window_proxy = self._plugin.ref_evince_proxy(self.out_gfile, self._window)
 
         elif not self.active and self.window_proxy is not None:
@@ -216,73 +228,81 @@ class SynctexViewHelper:
             for handler in self._view_active_handlers:
                 self._view.disconnect(handler)
 
-            self._window.get_data(WINDOW_DATA_KEY)._action_group.set_sensitive(False)
+            self._plugin._action_group.set_sensitive(False)
             self._plugin.unref_evince_proxy(self.out_gfile)
             self.window_proxy = None
 
 
-class SynctexWindowHelper:
-    def __init__(self, plugin, window):
-        self._window = window
-        self._plugin = plugin
+class SynctexWindowActivatable(GObject.Object, Gedit.WindowActivatable):
+    __gtype_name__ = "SynctexWindowActivatable"
 
+    window = GObject.property(type=Gedit.Window)
+    view_dict = {}
+    _proxy_dict = {}
+
+    def __init__(self):
+        GObject.Object.__init__(self)
+
+    def do_activate(self):
         self._insert_menu()
 
-        for view in window.get_views():
-            self.add_helper(view, window)
+        for view in self.window.get_views():
+            self.add_helper(view, self.window)
 
         self.handlers = [
-            window.connect("tab-added", lambda w, t: self.add_helper(t.get_view(),w, t)),
-            window.connect("tab-removed", lambda w, t: self.remove_helper(t.get_view())),
-            window.connect("active-tab-changed", self.on_active_tab_changed)
+            self.window.connect("tab-added", lambda window, tab: self.add_helper(tab.get_view(), window)),
+            self.window.connect("tab-removed", lambda window, tab: self.remove_helper(tab.get_view())),
+            self.window.connect("active-tab-changed", self.on_active_tab_changed)
         ]
+
+    def do_deactivate(self):
+        for h in self.handlers:
+            self.window.disconnect(h)
+
+        for view in self.window.get_views():
+            self.remove_helper(view)
+
+        self._remove_menu()
+
     def on_active_tab_changed(self, window,  tab):
         view_helper = tab.get_view().get_data(VIEW_DATA_KEY)
+
         if view_helper is None:
             active = False
         else:
-            active = view_helper.active 
+            active = view_helper.active
+
         self._action_group.set_sensitive(active)
 
-
     def add_helper(self, view, window):
-        helper = SynctexViewHelper(view, window, self._plugin)
+        helper = SynctexViewHelper(view, window, self)
         location = view.get_buffer().get_location()
 
         if location is not None:
-            SynctexPlugin.view_dict[location.get_uri()] = helper
+            self.view_dict[location.get_uri()] = helper
         view.set_data (VIEW_DATA_KEY, helper)
 
     def remove_helper(self, view):
         helper = view.get_data(VIEW_DATA_KEY)
+
         if helper.gfile is not None:
-            del SynctexPlugin.view_dict[helper.gfile.get_uri()]
+            del self.view_dict[helper.gfile.get_uri()]
+
         helper.deactivate()
         view.set_data(VIEW_DATA_KEY, None)
 
-    def __del__(self):
-        self._window = None
-        self._plugin = None
-
-    def deactivate(self):
-        for h in self.handlers:
-            self._window.disconnect(h)
-        for view in self._window.get_views():
-            self.remove_helper(view)
-        self._remove_menu()
-
     def _remove_menu(self):
-        manager = self._window.get_ui_manager()
+        manager = self.window.get_ui_manager()
         manager.remove_ui(self._ui_id)
         manager.remove_action_group(self._action_group)
         manager.ensure_update()
 
     def _insert_menu(self):
         # Get the GtkUIManager
-        manager = self._window.get_ui_manager()
+        manager = self.window.get_ui_manager()
 
         # Create a new action group
-        self._action_group = Gtk.ActionGroup(name="SynctexPluginActions")
+        self._action_group = Gtk.ActionGroup(name="SynctexWindowActivatableActions")
         self._action_group.add_actions([("SynctexForwardSearch", None,
                                         _("Forward Search"), "<Ctrl><Alt>F",
                                         _("Forward Search"), self.forward_search_cb)])
@@ -294,58 +314,43 @@ class SynctexWindowHelper:
         self._ui_id = manager.add_ui_from_string(ui_str)
 
     def forward_search_cb(self, action, what):
-        self._window.get_active_view().get_data(VIEW_DATA_KEY).sync_view(Gtk.get_current_event_time())
-
-class SynctexPlugin(GObject.Object, Gedit.WindowActivatable):
-    __gtype_name__ = "SynctexPlugin"
-
-    window = GObject.property(type=Gedit.Window)
-    view_dict = {}
-    _proxy_dict = {}
-
-    def __init__(self):
-        GObject.Object.__init__(self)
-
-    def do_activate(self):
-        window = self.window
-        helper = SynctexWindowHelper(self, window)
-        window.set_data(WINDOW_DATA_KEY, helper)
-
-    def do_deactivate(self):
-        window = self.window
-        window.get_data(WINDOW_DATA_KEY).deactivate()
-        window.set_data(WINDOW_DATA_KEY, None)
+        self.window.get_active_view().get_data(VIEW_DATA_KEY).sync_view(Gtk.get_current_event_time())
 
     def source_view_handler(self, out_gfile, uri_input, source_link, time):
 
-        if uri_input not in SynctexPlugin.view_dict:
-            window = SynctexPlugin._proxy_dict[out_gfile.get_uri()][2]
+        if uri_input not in self.view_dict:
+            window = self._proxy_dict[out_gfile.get_uri()][2]
+
             tab = window.create_tab_from_location(Gio.file_new_for_uri(uri_input),
                                                   None, source_link[0] - 1, 0, False, True)
+
             helper =  tab.get_view().get_data(VIEW_DATA_KEY)
             helper._goto_handler = tab.get_document().connect_object("loaded", 
-                                                SynctexViewHelper.goto_line_after_load, 
+                                                SynctexViewHelper.goto_line_after_load,
                                                 helper, source_link[0] - 1, time)
         else:
-            SynctexPlugin.view_dict[uri_input].goto_line(source_link[0] - 1, time)
+            self.view_dict[uri_input].goto_line(source_link[0] - 1, time)
 
     def ref_evince_proxy(self, gfile, window):
         uri = gfile.get_uri()
         proxy = None
-        if uri not in SynctexPlugin._proxy_dict:
+
+        if uri not in self._proxy_dict:
             proxy = EvinceWindowProxy (uri, True, _logger)
-            SynctexPlugin._proxy_dict[uri] = [1, proxy, window]
+            self._proxy_dict[uri] = [1, proxy, window]
             proxy.set_source_handler (lambda i, s, time: self.source_view_handler(gfile, i, s, time))
         else:
-            SynctexPlugin._proxy_dict[uri][0]+=1
-            proxy = SynctexPlugin._proxy_dict[uri][1]
+            self._proxy_dict[uri][0]+=1
+            proxy = self._proxy_dict[uri][1]
+
         return proxy
 
     def unref_evince_proxy(self, gfile):
         uri = gfile.get_uri()
-        if uri in SynctexPlugin._proxy_dict:
-            SynctexPlugin._proxy_dict[uri][0]-=1
-            if SynctexPlugin._proxy_dict[uri][0] == 0:
-                del SynctexPlugin._proxy_dict[uri]
+
+        if uri in self._proxy_dict:
+            self._proxy_dict[uri][0] -= 1
+            if self._proxy_dict[uri][0] == 0:
+                del self._proxy_dict[uri]
 
 # ex:ts=4:et:
